/*
  Java gfx-test framework

   Copyright (C) 2013  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;



import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.BitBltOperation;
import org.gfxtest.framework.annotations.BitBltOperations;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.COMMON_BITMAP)
@RenderStyle(RenderStyles.NORMAL)
@BitBltOperation(BitBltOperations.BITBLT)
@Transformation(Transformations.ROTATION)
@Zoom(1)
public class BitBltAffineRotateTransformOp extends BitBltAffineTransformOp
{
    private static double ROTATIONS[] = {0.0, Math.PI/10.0, -Math.PI/4, Math.PI/4, Math.PI, 2 * Math.PI, 3*Math.PI/2};
    private static final int NUM_ROTATIONS = ROTATIONS.length;

    private static final AffineTransform RotateTransformation[] = new AffineTransform[NUM_ROTATIONS];

    private static final RenderingHints RenderingHintsNearestNeighbor;
    private static final RenderingHints RenderingHintsBilinear;
    private static final RenderingHints RenderingHintsBicubic;

    private static AffineTransformOp RotateTransformationNearest1Op[] = new AffineTransformOp[NUM_ROTATIONS];
    private static AffineTransformOp RotateTransformationBilinear1Op[] = new AffineTransformOp[NUM_ROTATIONS];
    private static AffineTransformOp RotateTransformationBicubic1Op[] = new AffineTransformOp[NUM_ROTATIONS];

    private static AffineTransformOp RotateTransformationNearest2Op[] = new AffineTransformOp[NUM_ROTATIONS];
    private static AffineTransformOp RotateTransformationBilinear2Op[] = new AffineTransformOp[NUM_ROTATIONS];
    private static AffineTransformOp RotateTransformationBicubic2Op[] = new AffineTransformOp[NUM_ROTATIONS];

    static
    {
        RenderingHintsNearestNeighbor = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
        RenderingHintsBilinear = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        RenderingHintsBicubic = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);

        for (int i = 0; i < NUM_ROTATIONS; i++)
        {
            RotateTransformation[i] = AffineTransform.getRotateInstance(ROTATIONS[i]);

            RotateTransformationNearest1Op[i] = new AffineTransformOp(RotateTransformation[i], AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
            RotateTransformationBilinear1Op[i] = new AffineTransformOp(RotateTransformation[i], AffineTransformOp.TYPE_BILINEAR);
            RotateTransformationBicubic1Op[i] = new AffineTransformOp(RotateTransformation[i], AffineTransformOp.TYPE_BICUBIC);

            RotateTransformationNearest2Op[i] = new AffineTransformOp(RotateTransformation[i], RenderingHintsNearestNeighbor);
            RotateTransformationBilinear2Op[i] = new AffineTransformOp(RotateTransformation[i], RenderingHintsBilinear);
            RotateTransformationBicubic2Op[i] = new AffineTransformOp(RotateTransformation[i], RenderingHintsBicubic);
        }

    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRRotateTransformation0Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, RotateTransformationNearest1Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRRotateTransformation1Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, RotateTransformationNearest1Op[1]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRRotateTransformation2Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, RotateTransformationNearest1Op[2]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRRotateTransformation3Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, RotateTransformationNearest1Op[3]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRRotateTransformation4Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, RotateTransformationNearest1Op[4]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRRotateTransformation5Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, RotateTransformationNearest1Op[5]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRRotateTransformation6Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, RotateTransformationNearest1Op[6]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRRotateTransformation0Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGR(image, graphics2d, RotateTransformationNearest1Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRRotateTransformation1Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGR(image, graphics2d, RotateTransformationNearest1Op[1]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRRotateTransformation2Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGR(image, graphics2d, RotateTransformationNearest1Op[2]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRRotateTransformation3Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGR(image, graphics2d, RotateTransformationNearest1Op[3]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRRotateTransformation4Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGR(image, graphics2d, RotateTransformationNearest1Op[4]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRRotateTransformation5Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGR(image, graphics2d, RotateTransformationNearest1Op[5]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRRotateTransformation6Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGR(image, graphics2d, RotateTransformationNearest1Op[6]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB_PRE.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRPreRotateTransformation0Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGRPre(image, graphics2d, RotateTransformationNearest1Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB_PRE.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRPreRotateTransformation1Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGRPre(image, graphics2d, RotateTransformationNearest1Op[1]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB_PRE.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRPreRotateTransformation2Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGRPre(image, graphics2d, RotateTransformationNearest1Op[2]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB_PRE.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRPreRotateTransformation3Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGRPre(image, graphics2d, RotateTransformationNearest1Op[3]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB_PRE.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRPreRotateTransformation4Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGRPre(image, graphics2d, RotateTransformationNearest1Op[4]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB_PRE.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRPreRotateTransformation5Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGRPre(image, graphics2d, RotateTransformationNearest1Op[5]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_4BYTE_ARGB_PRE.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType4ByteABGRPreRotateTransformation6Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType4ByteABGRPre(image, graphics2d, RotateTransformationNearest1Op[6]);
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new BitBltAffineRotateTransformOp().runTestSuite(args);
    }
    
}
